@@ -2,7 +2,7 @@ from .pipeline import SampleDataContainer, get_manifest, run_pipeline
 from .preprocess import preprocess_noob
 from .raw_dataset import RawDataset, get_raw_datasets, get_array_type
 from .postprocess import consolidate_values_for_sheet
-from .read_geo_processed import read_geo
+from .read_geo_processed import read_geo, detect_header_pattern
 
 __all__ = [
     'RawDataset',
@@ -14,4 +14,5 @@ __all__ = [
     'consolidate_values_for_sheet',
     'get_array_type',
     'read_geo',
+    'detect_header_pattern',
 ]
